/*
 * block-nav-support.cpp for bwt tunneling
 * Copyright (c) 2018 Uwe Baier, Kadir Dede All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "block-nav-support.hpp"

#include <stack>

using namespace std;

//// CONSTRUCTION /////////////////////////////////////////////////////////////

void block_nav_support::compute_blocks() {
	//create helpful arrays
	vector<t_idx_t> bend; //block end
	vector<t_idx_t> &pbend = m_end; //perfect block end

	//initialize arrays and a stack
	bend.resize( blocks );
	pbend.resize( blocks );
	for (t_idx_t b = 0; b < blocks; b++) {
		bend[b] = bwtrs.run_lf(b);
		pbend[b] = bwtrs.run_lf(b);
	}
	stack<t_idx_t> s;

	//run algorithm (without collision computation)
	for (t_idx_t b = 0; b < blocks; b++) {
		if (bwtrs.height(b) >= mbh) {
			s.push( b );
			do {
				auto b_ = bwtrs.run_of( bend[s.top()] );
				if (bend[s.top()] + bwtrs.height(s.top()) <= bwtrs.end(b_)) {
					//stacktop block can be extended
					s.push( b_ );
				} else {
					//block on stacktop is maximal
					b_ = s.top();
					s.pop();

					if (!s.empty()) {
						//adapt end of new stacktop block
						bend[s.top()] = bend[b_] + (bend[s.top()] - bwtrs.start(b_));

						//adapt end of perfect blocks
						if (bwtrs.height(b_) == bwtrs.height(s.top())) {
							pbend[s.top()] = pbend[b_];
							//set b_'s perfect width to 1 to avoid overlapping blocks
							pbend[b_] = bwtrs.run_lf(b_);
						}
					}
				}
			} while (!s.empty());
		}
	}
}

//// COLLISION HANDLING ///////////////////////////////////////////////////////

void block_nav_support::set_end( t_idx_t b, t_idx_t e ) {
	m_end[b] = e;
}

